\\ Copyright 2019, 2020, 2021, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
default(parisizemax,1500*10^6);
read("../vecper.gp");
read("../vecper-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
OEIS_check_verbose = 1;

option_verbose = 0;
check_count = 0;


\\=============================================================================
\\ Generic Helpers

to_binary(n)=fromdigits(binary(n))*sign(n);
OEIS_check_func("A007088","to_binary");

gf_terms(g,n) = {
  if(g==0,return(vector(n)));
  my(x = variable(g),
     zeros = min(n,valuation(g,x)),
     v = Vec(g + O(x^n)));
  if(zeros>=0, concat(vector(zeros,i,0), v),
               v[-zeros+1 .. #v]);
}

contfrac_to_rational(v) = {
  \\ contfracpnqn() returns matrix [ p[n], p[n-1];
  \\                                 q[n], q[n-1] ]
  \\ where p[n]/q[n] is the last partial quotient,
  \\ and p[n-1],q[n-1] is the second last (unused here)
  my(m=contfracpnqn(v));
  m[1,1]/m[2,1];
}

\\=============================================================================

\\-----------------------------------------------------------------------------
\\ A253389  n^power pattern of second-last digit

{
  \\ start when 2 digits
  my(vv(n) =
     my(start=n); while(start<10,start*=n); start/=n;
     vecper_normalize
     (vecper_make_state(t->t=(t*n)%100;[t\10,t], start)));
  check_equal(vv(2),
              [[], [1,3,6,2,5,1,2,4,9,9,8,6,3,7,4,8,7,5,0,0]]);
  check_equal(vv(21),
              [[], [2,4,6,8,0]]);
  check_equal(vector(8,k,k--; 21^k%100),
              [1, 21, 41, 61, 81, 1, 21, 41]);

  check_equal(vv(14),
              [[1], [9, 4, 1, 2, 3, 0, 5, 8, 7, 6]]);
  check_equal(vector(8,k,k--; 14^k%100),
              [1, 14, 96, 44, 16, 24, 36, 4]);

  \\ check_equal(select(n->vecper_fixed_length(vv(n))>0,[2..120]),
  \\             []);


  my(a(n) = fromdigits(vecper_periodic(vv(n))));
  check_equal(vv(6), [[], [3, 1, 9, 7, 5]]);
  my(want=[13625124998637487500, 28428684442602686000, 1652983470, 2, 31975, 4400, 61964512293803548770, 8264462800, 0, 1234567890]);
  check_equal(vector(#want,n,n++; a(n)), want);

  check_equal(matrix(16,120,n,e,n++;e+=10; lift(Mod(n,100)^e)),
              matrix(16,120,n,e,n++;e+=10; n+=100; lift(Mod(n,100)^e)));

  for(n=2,100,
     my(len=vecper_periodic_length(vv(n)));
     20%len==0 || error());
  check_equal(vv(100), [[], [0]]);
  check_equal(vv(102),
              [[0], [0,0,1,3,6,2,5,1,2,4,9,9,8,6,3,7,4,8,7,5]]);
  if(0,
     for(n=2,1000,
        my(v=vv(n));
        if(vecper_periodic_length(v)>1 && vecper_get(v,1)==0, print(n)));
     quit);

  \\ FIXES in progress
  \\ OEIS_check_func("A253389",a,['offset,2]);
}


\\-----------------------------------------------------------------------------
\\ contfrac sqrt(3/2)
\\   A142238  convergents numerators
\\   A142239  convergents denominators

\\ continued fract 1, 4,2, 4,2, 4,2,
\\ cf A106469 =  1, 2,2, 4,2, 4,2, 4,2, ...

{
  my(sqrt6=quadgen(4*6));
  check_equal(simplify(sqrt6^2), 6);
  my(sqrt3on2=sqrt6/2);
  check_equal(simplify(sqrt3on2^2), 3/2);
  my(v=vecper_contfrac(sqrt3on2));
  check_equal(v,[[1], [4, 2]]);

  \\ A142238 convergent numerator
  my(A142238(n) = n>=0 || error();
     numerator(contfrac_to_rational(vecper_get_range(v,1,n+1))));
  OEIS_check_func("A142238",A142238,['offset,0]);

  \\ A142239 convergent denominator
  my(A142239(n) = denominator(contfrac_to_rational(vecper_get_range(v,1,n+1))));
  OEIS_check_func("A142239",A142239,['offset,0]);

}


\\-----------------------------------------------------------------------------
\\ period of low n digits of (3^x-1)/2

{
  if(0,
     my(vv(m) = vecper_make_state(t->t=(3*t+1)%m;[t,t], 0));
     for(n=1,8, my(m=10^n,v=vv(m)); print(apply(length,v)," ",m));
     quit);
}

\\-----------------------------------------------------------------------------
\\ A157697  sqrt(2/3)
\\   sqrt(6)/3
\\   2/sqrt(6)

A157697_quad = quadgen(6*4)/3;
A157697_vector(len) = digits(floor(A157697_quad*10^len));
{
  OEIS_check_vecfunc("A157697_vector");
}

\\-----------------------------------------------------------------------------
\\ A020763  1/sqrt(6)

A020763_quad = 1/quadgen(6*4);
A020763_vector(len) = digits(floor(A020763_quad*10^len));
{
  OEIS_check_vecfunc("A020763_vector");
  my(v=vecper_contfrac_of_xys(0,1/6,6));
  check_equal(v, [[0, 2], [2, 4]]);

  my(v=vecper_contfrac(A020763_quad));
  check_equal(v, [[0, 2], [2, 4]]);

  check_equal(A020763_quad, A157697_quad/2);

  \\ digits from floats
  \\ round to nearest in normal print
  if(0,
     for(len=1,20,
        default(realprecision,len);
        my(s=Str(1/sqrt(6)));
        my(t=Str("0.",floor(1/quadgen(6*4)*10^len)));
        if(s!=t,
           print(len);
           print("  ",s);
           print("  ",t));
     );
     quit);
        
}

\\-----------------------------------------------------------------------------
\\ A056055  decimals 1/n contains substring n

vector_contains_vector(v,sub) =
{
  for(i=1,#v-#sub+1, if(v[i..i+#sub-1]==sub, return(1)));
  0;
}
check_equal(vector_contains_vector([10,20,30,40],[30,40]),1);

vecper_contains_vector(v,sub) =
{
  for(i=1,vecper_fixed_length(v)+vecper_periodic_length(v),
     if(vecper_get_range(v,i,i+#sub-1)==sub, return(1)));
  0;
}

is_A056055_1n_substring_n(n) =
{
  n>=1 || return(0);
  my(v=vecper_frac_digits(1/n),
     d=digits(n));
  if(vecper_periodic(v)==[0],
     vector_contains_vector(vecper_fixed(v),d),
     vecper_contains_vector(v,d));
}
OEIS_check_predfunc("is_A056055_1n_substring_n");
\\ to a(900) = 14349
if(0,OEIS_check_predfunc("is_A056055_1n_substring_n",,'bfile));

\\-----------------------------------------------------------------------------
\\ A157665  729*n^2 - 1016*n + 354

A157665(n) =
{
  n>=1 || error();
  729*n^2 - 1016*n + 354;
}
{
  my(a=A157665);
  OEIS_check_func("A157665",a,['offset,1]);
  my(make_v(n) = [[27*n-19], [5, 2, 1, 1, 27*n-20,
                              1, 1, 2, 5, 54*n-38]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)),
              "make_v");

  my(xys=[0, 1/(39366*n - 27432),
          1129718145924*n^4 
          - 3148953734592*n^3 
          + 3291503216904*n^2 
          - 1529115641280*n + 266390176896]);
  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              xys);
  check_equal(xys[3] * xys[2]^2,
              729*n^2 - 1016*n + 354);
}


\\-----------------------------------------------------------------------------
\\ A158058  16*n^2 - 2*n

A158058(n) =
{
  n>=1 || error();
  16*n^2 - 2*n;
}
{
  my(a=A158058);
  OEIS_check_func("A158058",a,['offset,1]);
  my(make_v(n) = [[4*n-1], [1, 2, 1, 8*n-2]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)),
              "make_v");

  my(xys=[0, 1/8, 1024*n^2 - 128*n]);
  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              xys);
  check_equal(xys[3] * xys[2]^2,
              16*n^2 - 2*n);
}


\\-----------------------------------------------------------------------------
\\ A158062  36*n^2 - 2*n

A158062(n) =
{
  n>=1 || error();
  36*n^2 - 2*n;
}
{
  my(a=A158062);
  OEIS_check_func("A158062",a,['offset,1]);
  my(make_v(n) = [[6*n-1], [1, 4, 1, 12*n-2]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)),
              "make_v");

  my(xys=[0, 1/12, 5184*n^2 - 288*n]);
  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              xys);
  check_equal(xys[3] * xys[2]^2,
              36*n^2 - 2*n);
}


\\-----------------------------------------------------------------------------
\\ A007732 period length of 1/n, or 1

A007732_1n_period_or_1(n) =
{
  n>=1 || error();
  if(n==1,1,
    vecper_periodic_length(vecper_frac_digits(1/n)));
}
{
  my(a=A007732_1n_period_or_1);
  OEIS_check_func("A007732_1n_period_or_1",a,['offset,1]);
  \\ print(vector(100,n,n/a(n)));

  \\ prospective changes to A007732, seemed wrong
  check_equal(a(189), 6);
  check_equal(lcm([a(3),a(7)]), 6);
  check_equal(3^2, 9);
  check_equal(189, 3^3 * 7);
  check_equal(a(3), 1);
  check_equal(a(7), 6);
  my(ff(p,e) =
     e>=1 || error();
     if(p==2 || p==5, 0,
        p==3, if(e<=2,0,
                 e-3),
        e-1));

  \\ 54 == 2*3^3
  \\ 297 == 11 * 3^3
  \\ 297 want 6 got 2
  my(a_by_transform(n) = my(f=factor(n));
     my(ispow = setminus(f[,1]~,[2,3,5])==[]);
     lcm(apply(a,f[,1]~))
     * prod(i=1,#f[,1],
            my(p=f[i,1],e=f[i,2]);
            e == valuation(n,p) || error();
            my(F = if(p==2 || p==5, 0,
                      p==3, if(e<=2,0,
                               ispow, e-2, e-3),
                      e-1));
            F!='none || error();
            p^F));
  if(0,
     check_equal(vector(180,n,a(n)),
                 vector(180,n,a_by_transform(n))));
  if(0,
     for(n=1,oo,
        my(want=a(n), got=a_by_transform(n));
        if(got!=want,
           print(n" want "want" got "got);
           print(factor(n));
           quit)));
}


\\-----------------------------------------------------------------------------
\\ A049450  n*(3*n-1)

A049450(n) =
{
  n>=0 || error();
  n*(3*n-1);
}
{
  my(a=A049450);
  OEIS_check_func("A049450",a,['offset,0]);

  check_equal(vecper_contfrac_of_xys(0,1,27*a(0)),
              [[0], []]);
  check_equal(vecper_contfrac_of_xys(0,1,27*a(1)),
              [[7], [2, 1, 6, 1, 2, 14]]);

  my(make_v(n) = [[9*n-2],
                  [2, 2*n-1, 6, 2*n-1, 2, 18*n-4]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,27*a(n))),
              vector(100,n, make_v(n)),
              "make_v");

  my(xys=[0, 
          1/(192*n^2 - 64*n + 4),
          2985984*n^6 - 2985984*n^5 + 1119744*n^4 - 193536*n^3 + 15120*n^2 - 432*n]);
  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              xys);
  check_equal(xys[3] * xys[2]^2,
              27*n*(3*n-1));
}


\\-----------------------------------------------------------------------------
\\ A049451  n*(3*n+1)

A049451(n) =
{
  n>=0 || error();
  n*(3*n+1);
}
{
  my(a=A049451);
  OEIS_check_func("A049451",a,['offset,0]);

  check_equal(vecper_contfrac_of_xys(0,1,27*a(0)),
              [[0], []]);
  check_equal(vecper_contfrac_of_xys(0,1,27*a(1)),
              [[10], [2, 1, 1, 4, 1, 1, 2, 20]]);

  my(make_v(n) = [[9*n+1],
                  [2, 2*n-1, 1, 4, 1, 2*n-1, 2, 18*n+2]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,27*a(n))),
              vector(100,n, make_v(n)),
              "make_v");

  my(xys=[0,
          1/(192*n^2 + 64*n + 4),
          2985984*n^6 + 2985984*n^5 + 1119744*n^4 + 193536*n^3 + 15120*n^2 + 432*n]);
  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              xys);
  check_equal(xys[3] * xys[2]^2,
              27*n*(3*n+1));
}


\\-----------------------------------------------------------------------------
\\ A021386 = decimal digits of 1/382
{
  my(f=1/382,
     v=vecper_frac_digits(f),
     g=vecper_polrev(v,var='x));
  check_equal(gf_terms(g,7),
              [0, 0, 2, 6, 1, 7, 8]);
  check_equal(denominator(g),
              1 - x^95,
              "gf no cancel");
  check_equal(subst(g,x,1/10) / 10, f);
  if(0,
     recurrence_guess(vecper_get_range(v,1,1000));
     quit);
}


\\-----------------------------------------------------------------------------
\\ A010136  contfrac sqrt(46)

{
  my(v=vecper_contfrac_of_xys(0,1,46),
     a_vector(len) = vecper_get_range(v,1,len));
  OEIS_check_vecfunc("A010136",a_vector);

  my(C=binomial);
  check_equal(vector(10,n,n--; C(2*n,n)%2),
              vector(10,n,n--; n==0));
  my(a(n) =
(1/396)*(-346*(n % 12)+83*
               ((n+1) % 12)-49*((n+2) % 12)+17*
               ((n+3) % 12)+50*((n+4) % 12)+149*
               ((n+5) % 12)-115*((n+6) % 12)-16*
               ((n+7) % 12) +17*((n+8) % 12)+83*
               ((n+9) % 12)-49*((n+10) % 12)
               +380*((n+11) % 12))-6*(C(2*n,n) %
               2)
    );
  my(want=a_vector(120));
  check_equal(vector(#want,n,n--; a(n)), want);

  \\ convergents
  OEIS_check_vecfunc("A041078",
    len->contfracpnqn(a_vector(len),len)[1,]);
  OEIS_check_vecfunc("A041079",
    len->contfracpnqn(a_vector(len),len)[2,]);
}

\\-----------------------------------------------------------------------------
\\ A157110  1681*n^2 - 2606*n + 1010

A157110(n) = 1681*n^2 - 2606*n + 1010;
{
  my(a=A157110);
  OEIS_check_func("A157110",a,['offset,1]);

  my(make_v(n) = [[41*n-32], [4, 1, 1, 4, 82*n-64]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A080827
\\    1, 3, 5, 9, 13, 19, 25, 33, 41, 51,
\\    1  2  3  4   5
\\ recurrence_guess(concat(0,OEIS_data("A080827")))
\\   1/2 * n^2
\\   + vector_modulo([1, 1/2],n)

A080827(n) =
{
  n>=1 || error();
  sqr(n)>>1 + 1;
}


{
  my(a=A080827);
  OEIS_check_func("A080827",,['offset,1]);

  if(0,
  \\ (n^2/2 + 1)^2 == 1/4*n^4 +           n^2 + 1
  \\ ((n+1)^4+2)  ==      n^4 + 4*n^3 + 6*n^2 + 4*n + 3
  \\ sqrt((n+1)^4+2)
  \\ n=4 n+1=5
  \\ s=258 = 2*3*43
  \\ 145/9~ 16.11
  \\ s=quadgen(4*258)
  \\ s^2
  \\ bestappr(sqrt(258),9)
  \\ contfrac(sqrt(258)) \\ 16, 16, 32, 16
  my(n=4,
     s=n^4+2,
     v=vecper_contfrac_of_xys(0,1,s));
  print(vector(16,n, bestappr(sqrt(s),n)));
  check_equal(A080827(n), 9);
  check_equal(v, [[16], [16, 32]]);
  print();
  print("term ",A080827(n)" s="s);
  print(v);
  v=vecper_get_range(v,1,4);
  print(v);
  v=contfracpnqn(v,4);
  print(v);

  \\ check_equal(vector(10,n,a(n)),
  \\             vector(10,n,
  \\   my(s=n^4+2,
  \\      v=vecper_contfrac_of_xys(0,1,s));
  \\   v=vecper_get_range(v,1,2);
  \\   denominator(contfrac_to_rational(v))));
  quit);
}


\\-----------------------------------------------------------------------------
\\ A178308  (111 + sqrt(25277))/158
{
  \\ continued fraction is A105899
  my(v=vecper_contfrac_of_xys(111/158,1/158,25277));
  check_equal(v, [[], [1,1,2,2,3,3]],
              "contfrac purely periodic");
  OEIS_check_func("A105899",
                  (n) -> vecper_get(v,n+1),
                  ['offset,0]);
  OEIS_check_vecfunc("A105899",
    (len)->vecper_get_range(v,1,len));

  \\ convergents
  if(0,
     print(vector(10,n, numerator(contfrac_to_rational(vecper_get_range(v,1,n)))));
     print(vector(10,n, denominator(contfrac_to_rational(vecper_get_range(v,1,n)))));
     \\ not in OEIS: 1, 2, 5, 12, 41, 135, 176, 311, 798, 1907
     \\ not in OEIS: 1, 1, 3, 7, 24, 79, 103, 182, 467, 1116
     quit);

  \\ A178308 decimals
  \\ x = (111 + sqrt(25277))/158 
  \\ 158 *x -  111 = sqrt(25277)
  \\ (158 *x -  111)^2 - 25277 = 0
  my(p(x) = (158 *x -  111)^2 - 25277);
  my(sqrt25277 = quadgen(4*25277));
  check_equal(simplify(sqrt25277^2), 25277);
  my(c= (111 + sqrt25277)/158);
  check_equal(simplify(p(c)), 0);
  my(a_vector(len) = digits(floor(c*10^(len-1))));

  OEIS_check_vecfunc("A178308",a_vector);
  if(0, OEIS_check_vecfunc("A178308",a_vector,'bfile));
}


\\-----------------------------------------------------------------------------
\\ A156676  81*n^2 - 44*n + 6

A156676(n) =
{
  n>=0 || error();
  81*n^2 - 44*n + 6;
}
{
  my(a=A156676);
  OEIS_check_func("A156676",a,['offset,0]);

  my(make_v(n) = [[9*n-3],
         [1,1,3,1, 9*n-4, 1,3,1,1, 18*n-6]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));

  \\ n=0 value 6 wildly different
  check_equal(a(0), 6);
  check_equal(vecper_contfrac_of_xys(0,1,a(0)),
              [[2], [2, 4]]);
  my(v=make_v(0));
  check_equal(v, [[-3], [1,1,3,1,-4,1,3,1,1,-6]]);
  check_equal(vecper_contfrac_to_xys(v),
              [0, -1/396, 940896]);
  check_equal(core(940896), 6);
}


\\-----------------------------------------------------------------------------
\\ A157618  625*n^2 - 886*n + 314

A157618(n) =
{
  n>=1 || error();
  625*n^2 - 886*n + 314;
}
{
  my(a=A157618);
  OEIS_check_func("A157618",a,['offset,1]);

  my(make_v(n) = [[25*n-18], [3, 1, 1, 3, 50*n-36]]);
  check_equal(vecper_contfrac_of_xys(0,1,1042),
              [[32], [3, 1, 1, 3, 64]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A156719  144*n^2 - 127*n + 28

A156719(n) =
{
  n>=1 || error();
  144*n^2 - 127*n + 28;
}
{
  my(a=A156719);
  OEIS_check_func("A156719",a,['offset,1]);

  my(make_v(n) = [[12*n-6],
                  [1, 2, 2, 2, 1, 24*n-12]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A157443  121*n^2 - 38*n + 3

A157443(n) = 121*n^2 - 38*n + 3;
{
  my(a=A157443);
  OEIS_check_func("A157443",a,['offset,1]);

  my(make_v(n) = [[11*n-2],
     [3, 1, 1, 1, 11*n-3, 1, 1, 1, 3, 22*n-4]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A157621   625n^2 - 364n + 53

A157621(n) = 625*n^2 - 364*n + 53;
{
  my(a=A157621);
  OEIS_check_func("A157621",a,['offset,1]);

  my(make_v(n) = [[25*n-8],
                  [1, 2, 1, 1, 2, 1, 50*n-16]]);
  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));

  check_equal(simplify(vecper_contfrac_to_xys(make_v('n))),
              [0, 1/50, 1562500*n^2 - 910000*n + 132500]);
  check_equal((1/50)^2 * (1562500*n^2 - 910000*n + 132500),
              625*n^2 - 364*n + 53);
  check_equal(a(n), 625*n^2 - 364*n + 53);
}


\\-----------------------------------------------------------------------------
\\ A157440  121*n^2 - 204*n + 86

A157440(n) = 121*n^2 - 204*n + 86;
{
  my(a=A157440);
  OEIS_check_func("A157440",a,['offset,1]);

  my(make_v(n) = [[11*n-10],
    [1, 2, 1, 2, 11*n-10, 2, 1, 2, 1, 22*n-20]]);

  check_equal(vector(100,n,n++; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n++; make_v(n)));

  check_equal(make_v(1),
   [[1], [1,2, 1,2, 1,2, 1,2, 1,2]]);
}



\\-----------------------------------------------------------------------------
\\ A156640  169*n^2 + 140*n + 29

A156640(n) = 169*n^2 + 140*n + 29;
{
  my(a=A156640);
  check_equal(a(0), 29);
  OEIS_check_func("A156640",a,['offset,0]);

  my(make_v(n) = [[13*n+5], [2, 1, 1, 2, 26*n+10]]);

  check_equal(vector(100,n,n--; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n--; make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A154357  25*n^2 - 14*n + 2

A154357(n) = 25*n^2 - 14*n + 2;
\\ vector(20,n,4*A154357(n))
{
  my(a=A154357);
  OEIS_check_func("A154357",a,['offset,0]);

  my(make_v(n) = [[10*n-3],
                  [4, 1, 5*n-3, 1, 4, 20*n-6]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,4*a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A013656  n*(9*n-2)

A013656(n) = n*(9*n-2)
\\ vector(20,n,4*A013656(n))
{
  my(a=A013656);
  OEIS_check_func("A013656",a,['offset,0]);

  my(make_v(n) = [[6*n-1],
                  [3, 3*n-1, 3, 12*n-2 ]]);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,4*a(n))),
              vector(100,n, make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A157514   25*n^2 - n

A157514(n) = n>=1 || error(); 25*n^2 - n;
{
  my(a=A157514);
  OEIS_check_func("A157514",a,['offset,1]);

  my(make_v(n) = [[5*n-1], [1, 8, 1, 10*n-2]]);
  check_equal(vecper_contfrac_of_xys(0,1,a(1)),
              [[4], [1, 8]]);

  check_equal(vector(100,n,n++; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n++; make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A157446   16*n^2 - n

A157446(n) = n>=1 || error(); 16*n^2 - n;
{
  my(a=A157446);
  OEIS_check_func("A157446",a,['offset,1]);

  my(make_v(n) = [[4*n-1], [1, 6, 1, 8*n-2]]);
  check_equal(vecper_contfrac_of_xys(0,1,a(1)),
              [[3], [1, 6]]);

  check_equal(vector(100,n,n++; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n++; make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A154254  9n^2 - 8n + 2

A154254(n) = 9*n^2 - 8*n + 2;
{
  my(a=A154254);
  OEIS_check_func("A154254",a,['offset,0]);
  check_equal(a(0), 2);

  my(make_v(n) = [[3*n-2],
                  [1, 2, 3*n-2, 2, 1, 6*n-4]]);

  \\ n=0 gets -sqrt2 instead
  my(sqrt2=quadgen(4*2));
  check_equal(simplify(sqrt2^2), 2);
  check_equal(make_v(0),
              [[-2], [1, 2, -2, 2, 1, -4]]);
  check_equal(vecper_contfrac_to_quad(make_v(0)),
              - sqrt2);

  check_equal(make_v(1),
              [[1], [1, 2, 1, 2, 1, 2]]);
  check_equal(vecper_normalize(make_v(1)),
              [[1],[1, 2]]);

  check_equal(vector(100,n,n++; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n++; make_v(n)));
}


\\-----------------------------------------------------------------------------
\\ A005563  n*(n+2)

A005563(n) = n*(n+2);
{
  my(a=A005563);
  OEIS_check_func("A005563",a,['offset,0]);

  my(make_v(n) = [[n], [1,2*n]]);

  \\ n=0 gets divide by 0
  check_equal(a(0), 0);
  check_equal(make_v(0), [[0],[1,0]]);
  \\ check_equal(vecper_contfrac_to_quad(make_v(0)), 0);

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n, make_v(n)));
}

\\-----------------------------------------------------------------------------
\\ A002378  n*(n+1)  pronic

A002378(n) = n*(n+1);
{
  my(a=A002378);
  OEIS_check_func("A002378",a,['offset,0]);

  my(make_v(n) = [[n], [2,2*n]]);

  \\ n=0 gets divide by 0
  check_equal(a(0), 0);
  check_equal(make_v(0), [[0],[2,0]]);
  \\ check_equal(vecper_contfrac_to_quad(make_v(0)), 0);

  \\ n=1 collapse
  check_equal(make_v(1), [[1],[2,2]]);
  check_equal(vecper_normalize(make_v(1)),
                         [[1],[2]]);

  check_equal(vector(100,n,n++; vecper_contfrac_of_xys(0,1,a(n))),
              vector(100,n,n++; make_v(n)));
}

\\-----------------------------------------------------------------------------
\\ A003215   3*n*(n+1) + 1  hex centred

A003215(n) = 3*n*(n+1) + 1
\\ vector(20,n,3*A003215(n))
{
  my(a=A003215);
  OEIS_check_func("A003215",a,['offset,0]);
  
  if(0,
     for(n=1,20,
        print(vecper_contfrac_of_xys(0,1,6*a(n))));
     quit);

  my(make_v(n) = [[3*n+1],
                  [1, 1, 2*n, 1, 1, 6*n+2]]);
  \\ n=0 collapse
  check_equal(make_v(0), [[1], [1, 1,0,1, 1, 2]]);
  check_equal(vecper_normalize([[1], [1, 2, 1, 2]]),
                         [[1],[1,2]]);

  check_equal(vector(5,n,n++; vecper_contfrac_of_xys(0,1,3*a(n))),
              vector(5,n,n++; make_v(n)));
}

\\-----------------------------------------------------------------------------
\\ A356869  4 / sqrt(5)

{
  my(sqrt5=quadgen(20));
  check_equal(4 / sqrt5, 4/5 * sqrt5);
  my(v=vecper_contfrac_of_xys(0, 4/5, 5));
  check_equal(v, [[1], [1, 3, 1, 2]]);
}

\\-----------------------------------------------------------------------------
\\ A063657  floor(sqrt) = round(sqrt)
\\   contfrac starts with 1


sqrt_round(n) =
{
  n>=0 || error();
  my(r,s=sqrtint(n,&r)); s + (r>s);
}
alias(A000194_sqrt_round, sqrt_round);
OEIS_check_func("A000194_sqrt_round");

is_A063657(n) =
{
    my(v=vecper_contfrac_of_xys(0,1,n));
    \\ iferr(vecper_get(v,2),e,0) == 1
    vecper_fixed_length(v) == 1 || error();
    v=vecper_periodic(v); #v && v[1]==1
}

{
  OEIS_check_predfunc("is_A063657");

  check_equal(vector(1000,n,n--; is_A063657(n)),
              vector(1000,n,n--;
                     sqrtint(n) != sqrt_round(n)),
              "is_A063657() vs sqrt_round");
}

\\-----------------------------------------------------------------------------
\\ A020228  pseudoprimes base 100

is_A020228(n) = n>1 && Mod(100,n)^(n-1)==1 && !isprime(n);
{
  OEIS_check_predfunc("is_A020228");
  check_equal(is_A020228(1), 0);

  \\ purely periodic and len divides 2n-2
  check_equal(vector(500,n,
    my(v=vecper_frac_digits(1/n));
    n>1
    && !isprime(n)
    && vecper_fixed_length(v) == 0
    && (2*n-2) % vecper_periodic_length(v) == 0),
              vector(500,n, is_A020228(n)),
              "is_A020228() vs period length");
}

\\-----------------------------------------------------------------------------
\\ A007533 = (5n+1)^2 + 4n+1
\\   Contfracs of its sqrt

A007533(n) =
{
  n>=0 || error();
  sqr(5*n+1) + n<<2 + 1;
}
{
  my(a=A007533);
  OEIS_check_func("A007533",,['offset,0]);

  \\ formula by Bruno Berselli in A007533
  check_equal(vector(100,n,n--; a(n)),
              vector(100,n,n--; 25*n^2 + 14*n + 2));

  check_equal(vector(1,n,n--; vecper_contfrac_of_xys(0,1,A007533(n))),
              vector(1,n,n--; vecper_normalize([[5*n+1],[2, 2, 10*n+2]])));

  check_equal(vector(100,n, vecper_contfrac_of_xys(0,1,A007533(n))),
              vector(100,n, [[5*n+1],[2, 2, 10*n+2]]));
}

\\-----------------------------------------------------------------------------
\\ A178309 Sqrt 25277

{
  \\ A178309 decimal expansion
  my(want=OEIS_data("A178309"));
  check_equal(digits(sqrtint(25277*100^(#want-3))),
              want);
  if(0,
     \\ ~/OEIS/b178309.txt
     my(want=OEIS_data("A178309",'bfile));
     check_equal(#want,9998);
     check_equal(digits(sqrtint(25277*100^(#want-3))),
                 want));
}

\\-----------------------------------------------------------------------------
\\ Sqrt 157

{
  \\ A010206 Contrfrac sqrt 157
  my(v=vecper_contfrac_of_xys(0,1,157));
  OEIS_check_func("A010206",
                  (n) -> vecper_get(v,n+1));

  \\ A041288 convergent numerator
  my(A041288(n) = numerator(contfrac_to_rational(vecper_get_range(v,1,n))));
  OEIS_check_func("A041288",A041288,['offset,1]);

  \\ A041289 convergent denominator
  my(A041289(n) = denominator(contfrac_to_rational(vecper_get_range(v,1,n))));
  OEIS_check_func("A041289",A041289,['offset,1]);

  \\ A178310 Decimal expansion of sqrt(157).
  my(want=OEIS_data("A178310"));
  check_equal(digits(sqrtint(157*100^(#want-2))),
              want);
  if(0,
     \\ ~/OEIS/b178310.txt
     my(want=OEIS_data("A178310",'bfile));
     check_equal(#want,9999);
     check_equal(digits(sqrtint(157*100^(#want-2))),
                 want));
}

\\-----------------------------------------------------------------------------
\\ Fibonacci Mod

\\ periodic vector of Fibonacci() mod m
Fibonacci_mod_vecper(m,start0=0,start1=1) =
{
  vecper_make_state((state)->[lift(state[1]),
                              [state[2],vecsum(state)]],
                    [Mod(start0,m), Mod(start1,m)]);
}

A001175_Pisano_period(n) =
{
  vecper_periodic_length
    (vecper_normalize
       (Fibonacci_mod_vecper(n)));
}
{
  OEIS_check_func("A001175_Pisano_period");
}

\\ code by Charles R Greathouse in A001175
fibmod(n, m)=lift( ((Mod([1,1; 1,0], m))^n)[1,2] );
{
  check_equal(matrix(50,50,n,m, fibmod(n,m)),
              matrix(50,50,n,m, fibonacci(n)%m));
}


\\-----------------------------------------------------------------------------
\\ Lucas Numbers mod 4006272480

Lucas_mod_vecper(m) = Fibonacci_mod_vecper(m,2,1);
A106291_Lucas_period(n) =
{
  vecper_periodic_length
    (vecper_normalize
       (Lucas_mod_vecper(n)));
}
{
  OEIS_check_func("A106291_Lucas_period",,['offset,1]);

  my(n=4006272480,
     v=vecper_normalize(Lucas_mod_vecper(n)));
  check_equal(vecper_fixed_length(v), 0);
  v=vecper_periodic(v);

  my(from_code = [
1, 2, 3, 4, 9349, 103682, 7, 370248451,
    521, 99845763, 11, 3952253959, 4006271116, 228826127, 4006104719,
    4006208401, 18, 5778, 1860498, 599074578, 969323029, 15127, 20633239,
    4006272281, 4006272404, 1149851, 24476, 29, 87403803, 2207, 1368706081,
    439204, 1468551844, 1568397607, 12752043, 3010349, 47, 39603, 4003262131,
    2537720636, 4005833276, 4870847, 2637566399, 322, 33385282, 2737412162,
    4005122629, 4006248004, 199, 4006272451, 3985639241, 843, 76, 3036949451,
    64079, 167761, 271443, 1364, 4006272469, 4006271959, 54018521, 4006263131,
    4006272476, 3636024029, 4006272479, 7881196, 3864850156, 4006268909, 3571,
    141422324, 3998391284, 710647, 123
]);
  check_equal(#Set(v), 73);
  check_equal(#from_code, 73);
  \\ print(v); quit;
}


\\-----------------------------------------------------------------------------
\\ A355674  decimal 1351/780, above sqrt(3)

{
  my(f=1351/780,
     v=vecper_frac_digits(f/10),
     want=[1, 7, 3, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2, 0, 5, 1, 2, 8, 2],
     got=vecper_get_range(v,1,#want));
  check_equal(got, want);

  \\ linrec order
  my(g=vecper_polrev(v));
  check_equal(denominator(g),
              -x^5 + x^4 - x^3 + x^2 - x + 1,
              "gf denominator");
  check_equal(poldegree(denominator(g)), 5);
  check_equal(vecper_periodic_length(v), 6);
  check_equal(g,
              (5*x^7 - 10*x^6 + 10*x^5 - 5*x^4 + 5*x^3 - 3*x^2 + 6*x + 1)
              / (-x^5 + x^4 - x^3 + x^2 - x + 1),
              "gf");

  \\ example
  my(e= 1 732051282051282051282051282051282051282);
  check_equal(e, floor(f*10^logint(e,10)));
  check_equal(digits(e),
              vecper_get_range(v,1,#digits(e)));

  if(0,
     recurrence_guess(vecper_get_range(v,1,1000));
     \\ (1,-1,1,-1,1)
     quit);
}

\\-----------------------------------------------------------------------------
\\ A355674  decimal 265/153, below sqrt(3)

{
  my(f=265/153,
     v=vecper_frac_digits(f/10),
     want=[1, 7, 3, 2, 0, 2, 6, 1, 4, 3, 7, 9, 0, 8, 4, 9, 6, 7, 3, 2, 0, 2, 6, 1, 4, 3, 7, 9, 0, 8, 4, 9, 6, 7, 3, 2, 0, 2, 6, 1, 4, 3, 7, 9, 0, 8, 4, 9, 6, 7, 3, 2, 0, 2, 6, 1, 4, 3, 7, 9, 0, 8, 4, 9, 6, 7, 3, 2, 0, 2, 6, 1, 4, 3, 7, 9, 0, 8, 4, 9, 6, 7, 3, 2, 0, 2, 6, 1],
     got=vecper_get_range(v,1,#want));
  check_equal(got, want);

  \\ linrec order
  my(g=vecper_polrev(v));
  check_equal(denominator(g),  1 - x^16);
  check_equal(poldegree(denominator(g)), 16);
  check_equal(vecper_periodic_length(v), 16);

  \\ example
  my(e= 1 732026143790849673202614379084967320261);
  check_equal(e, floor(f*10^logint(e,10)));
  check_equal(digits(e),
              vecper_get_range(v,1,#digits(e)));

  if(0,
     recurrence_guess(vecper_get_range(v,1,1000));
     \\ (1,-1,1,-1,1)
     quit);
}

\\-----------------------------------------------------------------------------
\\ A210622  decimal 377/120, near pi

{
  my(f=377/120,
    v=vecper_frac_digits(f/10),
     want=OEIS_data("A210622"),
     got=vecper_get_range(v,1,#want));
  check_equal(got, want);
  my(g=vecper_polrev(v));
  check_equal(denominator(g), 1-x);
  check_equal(vecper_periodic_length(v), 1);
  if(0,
     recurrence_guess(vecper_get_range(v,1,1000));
     quit);
}


\\-----------------------------------------------------------------------------
\\ A210621  decimal 256/81, near pi

{
  my(f=256/81,
    v=vecper_frac_digits(frac(f)),
     want=OEIS_data("A210621"),
     got=concat(floor(f),vecper_get_range(v,1,#want-1)));
  check_equal(got, want);
  my(g=vecper_polrev(v));
  check_equal(denominator(g), 1-x^9);
  check_equal(vecper_periodic_length(v), 9);
  if(0,
     recurrence_guess(vecper_get_range(v,1,1000));
     quit);
}

\\-----------------------------------------------------------------------------
\\ Bits of 1/n

\\ A355202  array of 1/n bits
A355202(n,k) = bittest(1<<k\n,0);
{
  my(m=[
  0, 0, 0, 0, 0, 0, 0, 0;
  1, 0, 0, 0, 0, 0, 0, 0;
  0, 1, 0, 1, 0, 1, 0, 1;
  0, 1, 0, 0, 0, 0, 0, 0;
  0, 0, 1, 1, 0, 0, 1, 1;
  0, 0, 1, 0, 1, 0, 1, 0;
  0, 0, 1, 0, 0, 1, 0, 0;
  0, 0, 1, 0, 0, 0, 0, 0]);
  check_equal(matrix(matsize(m)[1],matsize(m)[2],n,k,
                     A355202(n,k)),
              m);

  my(want=[0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0]);
  check_equal(OEIS_antidiagonals_of_func(#want,A355202, 1,1),
              want,
              "A355202 DATA by anti func");

  \\ 1/7 example
  check_equal(to_binary(1<<12\7), 001001001001);

  \\ code by Chittaranjan Pardeshi in A355202
  my(T(n, k) = my(r=lift(Mod(2, n)^
              (k-1))); 2*r>=n;
    );
  check_equal(matrix(50,150,n,k, A355202(n,k)),
              matrix(50,150,n,k, T(n,k)));

  check_equal(matrix(50,150,n,k, 2*(2^(k-1)%n) >= n),
              matrix(50,150,n,k, A355202(n,k)));

  my(T2(n,k) = my(r=lift(Mod(2,n)^(k-1)));
               2*r >= n);
  check_equal(matrix(50,500,n,k, T2(n,k)),
              matrix(50,500,n,k, A355202(n,k)));

  \\ 2^k/n mod 2
  \\ (2^(k-1) mod n)*2 /n mod 2
  \\ vector(15,k, 2*(2^k%7) >= 7)
  \\ concat(0,binary(1<<16\7))
}

\\ A007733 period length of binary 1/n, or 1
A007733_1n_binary_period_or_1(n) = if(n==1,1,vecper_periodic_length(vecper_frac_digits(1/n,2)));
OEIS_check_func("A007733_1n_binary_period_or_1",,['offset,1]);

{
  \\ column sums
  \\ not in OEIS: 1, 2, 4, 10, 19, 42, 85, 176, 348, 702
  if(0,
     my(c(k) = sum(n=1,1000000, A355202(n,k)));
     print(vector(10,k,c(k)));
     quit);
}

\\-----------------------------------------------------------------------------
\\ Decimals of 1/n

\\ A355068 array of decimals

\\ fixed part of 1/n,
\\ make into single integer.
A114205_1n_fixed(n) = fromdigits(vecper_fixed(vecper_frac_digits(1/n)));
OEIS_check_func("A114205_1n_fixed");

\\ periodic part of 1/n,
\\ make into single integer.
A060284_1n_periodic(n) = if(n==1,0,fromdigits(vecper_periodic(vecper_frac_digits(1/n))));
OEIS_check_func("A060284_1n_periodic");

\\ A266385 fixed and one periodic of 1/n,
\\ make into single integer.
A266385_1n_both(n) =
{
  if(n==1,1,
     my(v=vecper_frac_digits(1/n));
     fromdigits(if(v[2]==[0],v[1],concat(v))));
}
OEIS_check_func("A266385_1n_both");

\\-----------------------------------------------------------------------------
\\ A051626 period length of 1/n, or 0
\\ make into single integer.
A051626_1n_period_or_0(n) =
{
  if(n==1,0,
     my(v=vecper_periodic(vecper_frac_digits(1/n)));
     if(v==[0], 0, #v));
}
OEIS_check_func("A051626_1n_period_or_0",,['offset,1]);

test_frac_digits_sans_0s(n) =
{
  if(n==1,[1],
     my(v=vecper_frac_digits(1/n),
        f=vecper_fixed(v),
        p=vecper_periodic(v));
     while(#f&&f[1]==0, f=f[^1]);
     if(p==[0],p=[]);
     concat(f,p));
}
check_equal(vecper_frac_digits(1/8), [[1, 2, 5], [0]]);
check_equal(test_frac_digits_sans_0s(8), [1,2,5]);
\\ not in OEIS: 1, 5, 3, 2, 5, 2, 1, 6, 1, 4, 2, 8

\\ A121341 length of fractional fixed and one periodic of 1/n
\\ n=1 1/1 = 1.00 is reconked no digits
A121341_1n_both_length(n) =
{
  if(n==1,0,
     my(v=vecper_frac_digits(1/n));
     vecper_fixed_length(v)
     + if(v[2]!=[0], vecper_periodic_length(v)));
}
OEIS_check_func("A121341_1n_both_length",,['offset,1]);

\\ A352153  smallest digit in 1/n
\\          excluding high 0s
A352153_1n_smallest_digit(n) = vecmin(test_frac_digits_sans_0s(n));
OEIS_check_func("A352153_1n_smallest_digit",,['offset,1]);
\\ select(n->A352153_1n_smallest_digit(n)==0,[2..100])
\\ not in OEIS: 11, 13, 17, 19, 21, 23, 27, 29, 31, 33, 34, 37

A333236_1n_largest_digit(n) = \
  vecmax(test_frac_digits_sans_0s(n));
OEIS_check_func("A333236_1n_largest_digit",,['offset,1]);

OEIS_check_func("A266385",(n)->fromdigits(test_frac_digits_sans_0s(n)), ['offset,1]);

{
  my(d(n) = if(n==1,[1],
               my(v=vecper_frac_digits(1/n));

               while(v = vecper_ensure_fixed(v,1);
                     #v[1] && v[1][1]==0,
                     v[1] = v[1][^1]);
               v=vecper_normalize(v);
               my(f=vecper_fixed(v),
                  p=vecper_periodic(v));
               if(p==[0], f,
                  concat(f,p))));
  check_equal(d(11),[9,0]);
  \\ print(vector(20,n,fromdigits(d(n)))); quit;
  \\ not in OEIS: 3, 25, 2, 16, 142857, 125, 1, 1, 90, 83
}

{
  if(0,
     for(n=2,100,
        my(v=vecper_frac_digits(1/n),
           f=vecper_fixed(v),
           p=vecper_periodic(v));
        while(#f&&f[1]==0, f=f[^1]);
        if(p==[0],p=[]);
        v=[f,p];
        v=vecper_normalize(v);
        f=vecper_fixed(v);
        p=vecper_periodic(v);
        if(#f && #p>2,
           print(n"  "f" "p)));
     quit);
}

{
  \\ period lengths in bases
  if(0,
     for(b=11,16,
        for(not = 0,0,
           print(vector(20,n,n+=5;
     if(n==1,not,
     my(v=vecper_periodic(vecper_frac_digits(1/n,b)));
     if(v==[0], not, #v)))));
        print());
     quit);
}


\\-----------------------------------------------------------------------------
\\ A354948 array 1/n base phi

phi=quadgen(5);

{
  \\ R vs Fib mod
  if(0,
     my(n=12345,
        v=vecper_normalize(vecper_frac_digits(1/n,phi)));
     print("n = ",n);
     if(#concat(v)>50,
        print("vecper lengths ",apply(length,v)),
        print(v));
     my(R=1/n);
     my(dels=Map());
     for(i=1,9000,
        my(f=fibonacci(i-2)%n,
           g=fibonacci(i-1)%n);
        my(del=[(real(R*n)-f)/n,(imag(R*n)-g)/n],
           fg=(f+g*phi)/n,
           fgs=[]);
        mapisdefined(dels,del,&fgs);
        mapput(dels,del,concat(fgs,fg));
        if(0,
        printf("i=%2d  %d  %2d,%-2d  [%2d,%-2d] %s %2d,%2d  del %d,%d   %.6f\n",
               i,vecper_get(v,i),
               f,g,
               real(R*n)%n,imag(R*n)%n,
               if(real(R*n)==f && imag(R*n)==g, "=", " "),
               real(R*n),imag(R*n),
               (real(R*n)-f)/n,(imag(R*n)-g)/n,
               \\ R*1.0
               (f+g*phi)/n*1.0
              );
           );
        (0 <= R && R < 1) || error();
         real(R*n)%n == f || error();
         imag(R*n)%n == g || error();
        R*=phi; if(R>=1, R--);
     );

     dels=Mat(dels);
     for(i=1,matsize(dels)[1],
        my(v=dels[i,2]);
        print(dels[i,1]," min ",vecmin(v)+0.0);
        print(dels[i,1]," max ",vecmax(v)+0.0));
     quit);
}
\\ A354948_T(n,k) =
\\ {
\\   n>=1 || error();
\\   k>=1 || error();
\\   if(n==1,0,
\\      my(t=1,period=0,d);
\\      while(k>=1,
\\            t*=phi;
\\            if(d=(t>=n), t-=n);
\\            period++; k--;
\\            if(t==1, k%=period));
\\      d);
\\ }

read("/home/gg/tux/web/temporary/A354948-recip-phi.gp");
A354948_T=T;
kill(T)

A354948_find_j_step(n,k) =
{
   my(m = Mod([1,1;1,0], n)^k,
      a = lift(m[2,2]),
      b = lift(m[1,2]),
      j = -1);
   until (a + b * phi < n,
       j++;
       a = (n+b-a)%n;
       b = (n+b-a)%n);
  [j, a, b];

  \\ for(s=0,oo,
  \\    my(a=fibonacci(k-s-1)%n,
  \\       b=fibonacci(k-s)  %n);
  \\    if(a+b*phi < n, return(s)));
}
{
  if(0,
     my(n=8941,
        period=A001175_Pisano_period(n));
     print("period ",period);
     my(total=0,count=0);
     for(k=1,period,
        my(s=A354948_find_j_step(n,k));
        \\ print("k="k" "A354948_T(n,k)" s="s);
        total+=s[1]; count++);
     print("mean ",total/count*1.0);
     quit);
}

{
  if(0,
     my(n=2);
     print(vector(20,k, A354948_T(n,k)));

     \\ formula using floor
     \\ my(fracpart(x) = x - floor(x));
     \\ my(T(n,k) = floor(phi * fracpart(phi^(k-1)/n))
     \\   );

     \\ formula using >=
     my(T(n,k) = (phi * (phi^(k-1) % n)) >= n);
     print(vector(20,k, T(n,k)));

     check_equal(T        (1,2), 1);
     check_equal(A354948_T(1,2), 0);

     check_equal(T(2,5),         0);
     check_equal(A354948_T(2,5), 1);

     \\ 3-phi = 1.381
     my(n=2,k=5);
     check_equal(phi^4,      2+3*phi);
     check_equal(phi^4 % n,  3*phi-4, "mod");
     check_equal(phi*(3*phi-4), 3-phi, "mul");
     check_equal(phi * (phi^(k-1) % n), 3-phi);
     quit);
}

A354948_T_row_by_vecper(n,len) =
{
  if(n==1,vector(len),
     my(v=vecper_frac_digits(1/n,phi));
     vecper_get_range(v,1,len));
}
{
  my(rows=20,
     cols=100,
     by_vecper=Mat(vectorv(rows,n,
                A354948_T_row_by_vecper(n,cols))));
  check_equal(matrix(rows,cols,n,k, A354948_T(n,k)),
              by_vecper);

  for(n=1,300,
     my(v=vecper_frac_digits(1/n,phi));
     v=vecper_normalize(v);
     vecper_frac_fromdigits(v,phi) == 1/n  || error();
     vecper_fixed_length(v) == 0 || error(v);
     vecper_periodic_length(v) == A001175_Pisano_period(n) || error();
     );
}
{
  my(want=[0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1]);
  check_equal(OEIS_antidiagonals_of_func(#want,A354948_T, 1,1),
              want,
              "DATA by anti func");

  my(got=List([]),
     d=1,n=1,k=1);
  while(#got < #want,
        if(n<1, d++;n=d;k=1);
        listput(got, A354948_T(n,k));
        n--;k++);
  got=Vec(got);
  check_equal(got,want,
              "DATA by list");
}
{
  \\ denom on dividing down
  my(denom(n) = my(t = 1/n);
     while(t>=1, t/=phi; k++);
     denominator(gcd(real(t), imag(t))));
  check_equal(vector(1000,n,n++; denom(n)),
              vector(1000,n,n++; n),
              "denom");
}
{
  my(lz(n) = my(c=0);
     while(!A354948_T(n,c+1),c++); c);
  check_equal(vector(1000,n,n+=3; floor(log(n)/log(phi))),
              vector(1000,n,n+=3; lz(n)),
              "lz");
  if(0,
     print(vector(30,n,n+=3; lz(n)));
     quit);
  \\ not in OEIS: 2, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7
}
{
  \\ example array
  if(0,
     printf("%%e       k=");
     for(k=1,8, printf("%d  ", k));
     print();
     for(n=1,8,
        printf("%%e   n=%d: ",n);
        for(k=1,8,
           printf(" %d,", A354948_T(n,k)));
        print());
        quit);
}

fibmod(n, m) = lift(((Mod([1, 1; 1, 0], m))^n)[1, 2]); \\ as from A001175

\\ code by Chittaranjan Pardeshi in A354948
A354948_T_with_fibmod(n, k) =
{
  my(t = 1/n);
  while (t >= 1, t /= phi; k = k + 1);
  my(i = k,
     d = denominator(gcd(real(t), imag(t))),
     a=0, b=0);
  until( a + b*phi < d,
        print("at i="i"  a="a" b="b" t="t" value ",a + b*phi+0.0" cf d="d);
        i = i - 1;
        a = (d * real(t) * fibmod(i-1, d) + d * imag(t) * fibmod(i+0, d)) % d;
        b = (d * real(t) * fibmod(i+0, d) + d * imag(t) * fibmod(i+1, d)) % d);

  t = (a + b*phi)/d;
  \\ print("a="a" b="b" t = ",t" decrase i="i);
  k -= i;
  for(i=1, k-1,
     t *= phi;
     \\ print("  digit to t="t" is "t>=1);
     if (t >= 1, t -= 1));
  \\ print("  final t="t" is "t*phi>=1);
  if(t*phi>=1, 1, 0);
}
\\ A354948_T_with_fibmod(5000,15000)

\\ revised code by Chittaranjan Pardeshi in A354948
A354948_T_with_floor(n,k) =
{
   my(m = Mod([1,1;1,0], n)^k,
      a = lift(m[2,2]),
      b = lift(m[1,2]),
      j = -1);
   until (a + b * phi < n,
       j++;
       a = (n+b-a)%n;
       b = (n+b-a)%n);
   my(fp = ( a + b * phi ) / n);
   for (i=1, j, fp *= phi; fp -= floor(fp));
   floor(phi*fp);
}
{
  my(rows=20,
     cols=150);
  check_equal(matrix(rows,cols,n,k, A354948_T(n,k)),
              matrix(rows,cols,n,k, A354948_T_with_floor(n,k)),
              "A354948_T_with_floor()");
}

\\ algorithm by Chittaranjan Pardeshi in A354948
A354948_T_with_algorithm(n,k) =
{
  max_R = -oo;
  max_a = -oo;
  min_a = oo;
  max_b = -oo;
  min_b = oo;
  my(a=fibonacci(-1),
     b=fibonacci(0));
  for(i=1,k-1,
     max_R = max(max_R, a + b*phi);
     max_a = max(max_a, a);
     if(a < min_a,
        min_a = min(min_a, a);
        \\ print("min a  ",min_a);
       );
     max_b = max(max_b, b);
     min_b = min(min_b, b);
     [a, b] = [b, b+a];
     if(a + b*phi >= n,
        a = a - n));
  [a, b] = [b, b+a];
  (a + b*phi >= n);
}

\\ simple code by Chittaranjan Pardeshi in A354948
A354948_T_by_simple(n,k) =
{
  if(n == 1, 0,
     my(t = 1/n, d = 0);
     for(i=1, k,
        t = t * phi;
        t -= (d = t >= 1));
     d);
}
{
  my(rows=20,
     cols=150);
  check_equal(matrix(rows,cols,n,k, A354948_T(n,k)),
              matrix(rows,cols,n,k, A354948_T_by_simple(n,k)),
              "A354948_T_with_floor()");
}

{
  \\ maximum R
  \\ suminf(k=0, 1/phi^(2*k+1)) \\ == 1
  \\
  \\ n = 1000
  \\ a = -1161 951
  \\ a = -440 951
  \\ n = 1001
  \\ a = -1144 935
  \\ a = -428 935

  if(0,
     my(n=1000000000);
     print("n = ",n);
     A354948_T_with_algorithm(n,n^2+1000);
     print("a = "min_a" "max_a);
     print("a = "min_b" "max_b);
     quit);
}
{
  my(rows=20,
     cols=150);
  check_equal(matrix(rows,cols,n,k,n++; A354948_T(n,k)),
              matrix(rows,cols,n,k,n++; A354948_T_with_algorithm(n,k)),
              "A354948_T_with_algorithm()");
}
{
  if(0,
     my(seen(n,k) =
        my(a=fibonacci(-1),
           b=fibonacci(0),
           m=Map());
        for(i=1,k-1,
           mapput(m,[a,b],1);
           -2*n < a  || error();
           a < n   || error();
           -n < b  || error();
           b < n   || error();
           [a, b] = [b, b+a];
           if(a + b*phi >= n,
              a = a - n));
        m);
     for(n=2,20,
        my(m=seen(n, 10*n^2),
           acomb = #[-2*n+1 .. n-1],
           bcomb = #[-n+1 .. n-1],
           scomb = #[0 .. n-1],
          prob=acomb*bcomb/scomb);
        print(n" "#m));
     quit);
}





\\-----------------------------------------------------------------------------
\\ A354466 sum 1/d digits same decimals

is(n) =
{
  my(v=digits(n));
  vecmin(v) || return(0);
  my(f=sum(i=1,#v, 10/v[i]));
  \\ print(f" "f*1.0" "logint(floor(f),10));
  n == floor(f*10^(#v-1-logint(floor(f),10)));
}
\\ is(28428571)
\\ quit
\\ select(is,[1..200])
{
  my(n=825);
  my(f=1/8 + 1/2 + 1/5);
  check_equal(f,  33/40, "n=825 f");
  check_equal(f == .825, 1);
  check_equal(floor(f*1000), n);
  check_equal(#digits(n), 3);
  \\ 33/40.
}
{
  my(n=153);
  my(f=1/1 + 1/5 + 1/3);
  check_equal(f,  23/15);
  check_equal(floor(f*100), n);
  check_equal(#digits(n), 3);
}
{
  if(0,
     my(want=[1, 13, 145, 153, 1825, 15789, 16666, 21583, 216666, 2416666, 28428571]);
     want=want[1..6];
     for(n=1,want[#want],
        my(got=is(n),
           want=(setsearch(want,n)!=0));
        if(got!=want,
           print("n="n" got="got" want="want))));
}


\\-----------------------------------------------------------------------------
\\ A096494 Contfrac Prime Largest Periodic Term

A096494_prime_contfrac_periodic_largest(n) =
{
  vecmax(vecper_periodic
    (vecper_contfrac_of_xys(0,1,prime(n))));
}
{
  OEIS_check_func("A096494_prime_contfrac_periodic_largest",,['offset,1]);
}


\\-----------------------------------------------------------------------------
\\ A322289 Primes Sort by Contfrac

{
  check_equal(vecper_contfrac_of_xys(0,1,2), [[1], [2]]);
  check_equal(vecper_contfrac_of_xys(0,1,3), [[1], [1, 2]]);
  check_equal(vecper_contfrac_of_xys(0,1,5), [[2], [4]]);

  \\ u and v are vecors, return comparison lex()
  \\ first by vecmax and if equal by geometric mean
  my(cmpfunc(u,v) =
     my(c=lex(vecmax(u),vecmax(v)));
     if(c,return(c));
     my(e=lcm(#u,#v));
     lex(vecprod(u)^(e/#u), vecprod(v)^(e/#v)));

  \\ p == 1 mod 4 use (1+sqrt(p))/2
  \\   != 1 mod 4 use sqrt(p)
  my(want=OEIS_data("A322289"));
  want=want[1..10];
  my(P=primes(20*#want),
     pers=apply(p->vecper_periodic
      (if(p%4==1, vecper_contfrac_of_xys(1/2,1/2,p),
          vecper_contfrac_of_xys(0,1,p))), P));

  my(S = vecextract(P,vecsort(pers,cmpfunc,1)[1..#want]));
  check_equal(S, want);

  if(0,
     \\ largest part in the s and (1+s)/2 forms
     print(apply(v->vecmax(v),pers[4..14]));
     quit);
  \\ not in OEIS: 4, 6, 3, 3, 8, 8, 5, 10, 5, 5, 12
}

\\-----------------------------------------------------------------------------
\\ Continued Fraction of sqrt(prime(n)) Array

{
  my(T(n,k) =
     my(v=vecper_contfrac_of_xys(0,1,prime(n)));
     vecper_get(v,k));
  my(row(n) = vector(n,k, T(n-k+1,k)));

  \\ print(concat(vector(6,n,row(n))));  quit;
  \\ not in OEIS: 1, 1, 2, 2, 1, 2, 2, 4, 2, 2, 3, 1, 4, 1, 2, 3, 3, 1, 4, 2, 2
  \\ print(concat(vector(6,n,Vecrev(row(n)))));  quit;
  \\ not in OEIS: 1, 2, 1, 2, 1, 2, 2, 2, 4, 2, 2, 1, 4, 1, 3, 2, 2, 4, 1, 3, 3

  \\ second term of continued fraction sqrt(prime(n))
  OEIS_check_func("A067614", n->T(n,2));

  \\ matrix(5,12,n,k, T(n,k))
}

\\-----------------------------------------------------------------------------
\\ A077625 Largest Term  Contrfrac of sqrt(2^n-1)

A077625(n) =
{
  my(v=vecper_periodic(vecper_contfrac_of_xys(0,1,2^n-1)));
  if(#v, vecmax(v), 0);
}
{
  my(n=1);
  check_equal(2^n-1, 1);
  check_equal(issquare(1), 1);
  check_equal(A077625(1), 0);
}
{
  my(want=OEIS_data("A077625"));
  for(n=1,#want,
     \\ print("n="n);
     if(n>40,break);  \\ bit slow as n increases
     check_equal(A077625(n), want[n]));
}


\\-----------------------------------------------------------------------------
\\ A010204 Contrfrac sqrt 153
{
  my(v=vecper_contfrac_of_xys(0,1,153));
  OEIS_check_func("A010204",
                  (n) -> vecper_get(v,n+1));
}
\\ digits(sqrtint(153*100^20))
\\ sqrt(153)
\\ not in OEIS: 6, 9, 3, 1, 6, 8, 7, 6, 8, 5, 2, 9, 8


\\-----------------------------------------------------------------------------
\\ vecper_contfrac_of_xys() Various OEIS Sequences

\\ A121339, row vector of periodic part
A121339_contfrac_sqrt_periodic(n) = \
  vecper_periodic(vecper_contfrac_of_xys(0,1,n));
OEIS_check_func("A121339_contfrac_sqrt_periodic",,['offset,1]);

\\ A003285
A003285_contfrac_sqrt_periodic_length(n) = \
  vecper_periodic_length(vecper_contfrac_of_xys(0,1,n));
OEIS_check_func("A003285_contfrac_sqrt_periodic_length",,['offset,1]);

\\ A013644
A013644_contfrac_sqrt_period_4(n) = \
  vecper_periodic_length(vecper_contfrac_of_xys(0,1,n))==4;
{
  OEIS_check_predfunc("A013644_contfrac_sqrt_period_4");
}

\\ A013648 periodic part of contfrac of sqrt(n) has eactly one 1 term (and any amount of others)
A013648_contfrac_sqrt_periodic_one_1(n) =
{
  if(n==1,0,
     my(v=vecper_periodic(vecper_contfrac_of_xys(0,1,n)));
     sum(i=1,#v, v[i]==1) == 1);
}
{
  OEIS_check_predfunc("A013648_contfrac_sqrt_periodic_one_1");
}

\\-----------------------------------------------------------------------------
\\ vecper_contfrac_of_xys() random
\\
\\ These can use a lot of working space even for modest size num/den.
\\

test_xys_normalize(xys) =
{
  my(r, x,y,s); [x,y,s]=xys;
  s>=0 || error();
  s*=y^2; y=sign(y);
  if(issquare(s,&r), y*=r; s=1);
  if(s==1, x+=y;s=0);
  if(s==0||y==0, s=y=0);
  [x,y,s];
}
test_xys_equal(xysA, xysB) = \
  test_xys_normalize(xysA) == test_xys_normalize(xysB);
check_equal(test_xys_equal([-46/27,-23/38,1], [-2369/1026,0,0]), 1);

{
  \\ -13/12 - 17/18*sqrt(11/4)
  \\ -13/12 -1/159985347897613351004640 * sqrt(62783561259077837526989988615620562623859230400)
  check_equal(denominator([-13/12, -17/18]), 36);
  my(v=[[-3,2], [1,5,1,4,3,24,6,1,55,1,1,9,1,1,1,4,3,57,1,2,6,1,2,4,1,4,1,5,3,2,225,10,6,1,12,14,2,2,1,1]]);
  check_equal(vecper_contfrac_of_xys(-13/12, -17/18, 11/4), v);
  my(xys_again=[-13/12, -1/159985347897613351004640, 62783561259077837526989988615620562623859230400]);
  check_equal(vecper_contfrac_to_xys(v), xys_again);
  check_equal(core(xys_again[3]), 11);
  check_equal(xys_again[2]^2*xys_again[3], 3179/1296);
  check_equal(test_xys_normalize(xys_again), [-13/12, -1, 3179/1296]);
  check_equal(test_xys_equal(xys_again,[-13/12, -17/18, 11/4]), 1,
              "test_xys_equal()");
}

test_random_frac(limit,s) = \
  random(limit)/(random(limit)+1) * if(s&&random(2), -1, 1);
\\ test_random_signed(limit) = random(limit) * (-1)^random(2);
{
  if(0,
     my(longest=0);
     for(rep=1,1000,
        my(x=test_random_frac(15,1),
           y=test_random_frac(15,1),
           s=test_random_frac(15,0));
     \\ print("x="x" y="y" s="s"  factor "factor(s));
        my(v=vecper_contfrac_of_xys(x,y,s),
           xys_again=vecper_contfrac_to_xys(v));
     \\ longest=max(longest,vecper_fixed_length(v));
        longest=max(longest,#v[1]+#v[2]);
        my(got=test_xys_equal([x,y,s],xys_again));
        if(!got, \\ || issquare(s),
           print("x="x" y="y" s="s"   "factor(s));
           print("v="v);
           print("xys_again="xys_again);
           print("  norm ",vecper_normalize(v));
           print("  norm ",test_xys_normalize([x,y,s]));
           print("  norm ",test_xys_normalize(xys_again));
           print("  approx ",x+y*sqrt(s));
           print("  approx ",xys_again[1] + xys_again[2]*sqrt(xys_again[3]));
          );
        check_equal(got, 1);

        my(v_norm=vecper_normalize(v));
        if(v!=v_norm,
           print("not normalized  x="x" y="y" s="s"   "factor(s));
           print("v   ="v);
           print("norm="v_norm));
     );
     print("longest "longest);

     quit);
}

  \\ for(rep=1,1,
  \\    my(len1=random(6),
  \\       len2=random(8)+1,
  \\       fixed=vector(len1,i, random(10)+1),
  \\       periodic=vector(len1,i, random(10)+1));
  \\    if(!periodic, periodi[1]=1);
  \\    my(v=[fixed,periodic]);
  \\    my(xys=vecper_contfrac_of_xys(v),


\\-----------------------------------------------------------------------------
\\ A356070  au / light year
\\    63241.077...

{
  \\ check no cancel gf num/den so recurrence order
  \\ is period_length = 568524

  my(rat = 431996825232/6830953,
     num = 526559,
     den = 6830953,
     fra = num/den);
  check_equal(fra, rat - 63241);

  my(period_length = 568524,
     digs=digits(floor(fra*10^period_length)));
  digs=concat(vector(period_length-#digs),digs);
  check_equal(#digs,period_length);
  check_equal(fromdigits(digs) / (10^period_length-1),
              fra,
              "digs to fra");
  \\ my(v=vecper_frac_digits(fra));
  my(v=[[],digs]);
  check_equal(v, vecper_normalize(v),
              "normalize is nothing shorter");
  check_equal(vecper_frac_fromdigits(v), fra,
              "A356070 vecper fromdigits");

  my(gnum = Polrev(digs),
     gden = 1-x^period_length);
  my(g=gnum/gden);
  check_equal(poldegree(denominator(g)), period_length,
             "A356070 poldegree is period_length");
  check_equal(subst(g*x, x, 1/10), fra,
              "A356070 g subst is fra");
}


\\-----------------------------------------------------------------------------

check_equal(a,'a, "global variable a");
check_equal(b,'b, "global variable b");
check_equal(c,'c, "global variable c");
check_equal(d,'d, "global variable d");
check_equal(f,'f, "global variable f");
check_equal(g,'g, "global variable g");
check_equal(j,'j, "global variable j");
check_equal(m,'m, "global variable m");
check_equal(n,'n, "global variable s");
check_equal(r,'r, "global variable r");
check_equal(s,'s, "global variable s");
check_equal(t,'t, "global variable t");
check_equal(v,'v, "global variable v");
print("end");
